@@ -77,23 +77,12 @@ int flb_gzip_compress(void *in_data, size_t in_len,
     z_stream strm;
     mz_ulong crc;
 
-
     /*
-     * GZIP relies on an algorithm with worst-case expansion
-     * of 5 bytes per 32KB data. This means we need to create a variable
-     * length output, that depends on the input length.
-     * See RFC 1951 for details.
+     * Calculating the upper bound for a gzip compression is
+     * non-trivial, so we rely on miniz's own calculation
+     * to guarantee memory safety.
      */
-    int max_input_expansion = ((int)(in_len / 32000) + 1) * 5;
-
-    /*
-     * Max compressed size is equal to sum of:
-     *   10 byte header
-     *   8 byte foot
-     *   max input expansion
-     *   size of input
-     */
-    out_size = 10 + 8 + max_input_expansion + in_len;
+    out_size = compressBound(in_len);
     out_buf = flb_malloc(out_size);
 
     if (!out_buf) {
