@@ -143,6 +143,23 @@ int flb_router_connect(struct flb_input_instance *in,
     return 0;
 }
 
+int flb_router_connect_direct(struct flb_input_instance *in,
+                              struct flb_output_instance *out)
+{
+    struct flb_router_path *p;
+
+    p = flb_malloc(sizeof(struct flb_router_path));
+    if (!p) {
+        flb_errno();
+        return -1;
+    }
+
+    p->ins = out;
+    mk_list_add(&p->_head, &in->routes_direct);
+
+    return 0;
+}
+
 /*
  * This routine defines static routes for the plugins that have registered
  * tags. It check where data should go before the service start running, each
@@ -199,6 +216,7 @@ int flb_router_io_set(struct flb_config *config)
             continue;
         }
 
+
         flb_trace("[router] input=%s tag=%s", i_ins->name, i_ins->tag);
 
         /* Try to find a match with output instances */
@@ -224,16 +242,16 @@ int flb_router_io_set(struct flb_config *config)
 
                 if (!flb_router_match_type(i_ins->event_type, o_ins)) {
                     if (i_ins->event_type == FLB_INPUT_LOGS) {
-                        flb_error("[router] data generated by %s input are logs, "
+                        flb_debug("[router] data generated by %s input are logs, "
                                   "but matching destination plugin %s don't handle "
-                                  "logs",
+                                  "logs. Skipping destination.",
                                   flb_input_name(i_ins),
                                   flb_output_name(o_ins));
                     }
                     else if (i_ins->event_type == FLB_INPUT_METRICS) {
-                        flb_error("[router] data generated by %s input are metrics, "
+                        flb_debug("[router] data generated by %s input are metrics, "
                                   "but matching destination plugin %s don't handle "
-                                  "metrics",
+                                  "metrics.Skipping destination.",
                                   flb_input_name(i_ins),
                                   flb_output_name(o_ins));
                     }
@@ -270,5 +288,12 @@ void flb_router_exit(struct flb_config *config)
             mk_list_del(&r->_head);
             flb_free(r);
         }
+
+        /* Iterate instance routes direct */
+        mk_list_foreach_safe(r_head, r_tmp, &in->routes_direct) {
+            r = mk_list_entry(r_head, struct flb_router_path, _head);
+            mk_list_del(&r->_head);
+            flb_free(r);
+        }
     }
 }
