@@ -40,6 +40,12 @@ struct flb_config_map upstream_net[] = {
      "Select the primary DNS connection type (TCP or UDP)"
     },
 
+    {
+     FLB_CONFIG_MAP_STR, "net.dns.resolver", NULL,
+     0, FLB_TRUE, offsetof(struct flb_net_setup, dns_resolver),
+     "Select the primary DNS resolver type (LEGACY or ASYNC)"
+    },
+
     {
      FLB_CONFIG_MAP_BOOL, "net.keepalive", "true",
      0, FLB_TRUE, offsetof(struct flb_net_setup, keepalive),
@@ -106,14 +112,17 @@ struct mk_list *flb_upstream_get_config_map(struct flb_config *config)
      * flb_net_setup structure (and not lose it when flb_output_upstream_set overwrites
      * the structure) we need to do it this way (or at least that's what I think)
      */
-    if (config->dns_mode != NULL) {
-
-        for (config_index = 0 ; upstream_net[config_index].name != NULL ; config_index++) {
-            if(strcmp(upstream_net[config_index].name, "net.dns.mode") == 0)
-            {
+    for (config_index = 0 ; upstream_net[config_index].name != NULL ; config_index++) {
+        if (config->dns_mode != NULL) {
+            if (strcmp(upstream_net[config_index].name, "net.dns.mode") == 0) {
                 upstream_net[config_index].def_value = config->dns_mode;
             }
         }
+        if (config->dns_resolver != NULL) {
+            if (strcmp(upstream_net[config_index].name, "net.dns.resolver") == 0) {
+                upstream_net[config_index].def_value = config->dns_resolver;
+            }
+        }
     }
 
     config_map = flb_config_map_create(config, upstream_net);
@@ -209,6 +218,7 @@ struct flb_upstream *flb_upstream_create(struct flb_config *config,
         flb_errno();
         return NULL;
     }
+    u->config = config;
 
     /* Set default networking setup values */
     flb_net_setup_init(&u->net);
@@ -254,7 +264,6 @@ struct flb_upstream *flb_upstream_create(struct flb_config *config,
     u->flags         |= FLB_IO_ASYNC;
     u->thread_safe    = FLB_FALSE;
 
-
     /* Initialize queues */
     flb_upstream_queue_init(&u->queue);
 
@@ -429,15 +438,19 @@ static int prepare_destroy_conn(struct flb_upstream_conn *u_conn)
 static inline int prepare_destroy_conn_safe(struct flb_upstream_conn *u_conn)
 {
     int ret;
+    int locked = FLB_FALSE;
     struct flb_upstream *u = u_conn->u;
 
     if (u->thread_safe == FLB_TRUE) {
-        pthread_mutex_lock(&u->mutex_lists);
+        ret = pthread_mutex_trylock(&u->mutex_lists);
+        if (ret == 0) {
+            locked = FLB_TRUE;
+        }
     }
 
     ret = prepare_destroy_conn(u_conn);
 
-    if (u->thread_safe == FLB_TRUE) {
+    if (locked) {
         pthread_mutex_unlock(&u->mutex_lists);
     }
 
@@ -791,10 +804,13 @@ int flb_upstream_conn_timeouts(struct mk_list *list)
                 u_conn->ts_connect_timeout > 0 &&
                 u_conn->ts_connect_timeout <= now) {
                 drop = FLB_TRUE;
-                flb_error("[upstream] connection #%i to %s:%i timed out after "
-                          "%i seconds",
-                          u_conn->fd,
-                          u->tcp_host, u->tcp_port, u->net.connect_timeout);
+
+                if (!flb_upstream_is_shutting_down(u)) {
+                    flb_error("[upstream] connection #%i to %s:%i timed out after "
+                              "%i seconds",
+                              u_conn->fd,
+                              u->tcp_host, u->tcp_port, u->net.connect_timeout);
+                }
             }
 
             if (drop == FLB_TRUE) {
